@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/api.lisp,v 1.64 2006/07/16 13:32:59 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/api.lisp,v 1.70 2006/10/12 06:24:41 edi Exp $
 
 ;;; The external API for creating and using scanners.
 
@@ -185,6 +185,7 @@ NIL the function is allowed to destructively modify its first argument
                                                                      single-line-mode
                                                                      extended-mode
                                                                      destructive)
+  (declare #.*standard-optimize-settings*)
   (declare (ignore destructive))
   (when (or case-insensitive-mode multi-line-mode single-line-mode extended-mode)
     (signal-ppcre-invocation-error
@@ -197,6 +198,7 @@ NIL the function is allowed to destructively modify its first argument
                                                single-line-mode
                                                extended-mode
                                                destructive)
+  (declare #.*standard-optimize-settings*)
   (declare (ignore destructive))
   (excl:compile-re parse-tree
                    :case-fold case-insensitive-mode
@@ -256,6 +258,7 @@ internal purposes."))
                                 &key (start 0)
                                      (end (length target-string))
                                      ((:real-start-pos *real-start-pos*) nil))
+  (declare #.*standard-optimize-settings*)
   (when (< end start)
     (return-from scan nil))
   (let ((results (multiple-value-list (excl:match-re parse-tree target-string
@@ -401,7 +404,7 @@ declarations."
                   (scan ,(cond ((constantp regex env) regex)
                                (t scanner))
                         ,target-string :start ,%start :end ,%end
-                                       :real-start-pos ,%start)
+                                       :real-start-pos (or ,start 0))
                 ;; declare the variables to be IGNORABLE to prevent the
                 ;; compiler from issuing warnings
                 (declare
@@ -752,14 +755,17 @@ S-expression."))
 
 #-:cormanlisp
 (defmethod build-replacement-template ((replacement-function function))
+  (declare #.*standard-optimize-settings*)
   (list replacement-function))
 
 #-:cormanlisp
 (defmethod build-replacement-template ((replacement-function-symbol symbol))
+  (declare #.*standard-optimize-settings*)
   (list replacement-function-symbol))
         
 #-:cormanlisp
 (defmethod build-replacement-template ((replacement-list list))
+  (declare #.*standard-optimize-settings*)
   replacement-list)
 
 ;;; Corman Lisp's methods can't be closures... :(
@@ -822,7 +828,7 @@ S-expression."))
   (declare #.*standard-optimize-settings*)
   "Accepts a replacement template and the current values from the
 matching process in REGEX-REPLACE or REGEX-REPLACE-ALL and returns the
-corresponding template."
+corresponding string."
   ;; the upper exclusive bound of the register numbers in the regular
   ;; expression
   (let ((reg-bound (if reg-starts
@@ -950,7 +956,7 @@ representing the corresponding register start and end positions."
                             (end (length target-string))
                             preserve-case
                             simple-calls
-                            (element-type 'character))
+                            (element-type #+:lispworks 'lw:simple-char #-:lispworks 'character))
   (declare #.*standard-optimize-settings*)
   "Try to match TARGET-STRING between START and END against REGEX and
 replace the first match with REPLACEMENT.
@@ -1004,7 +1010,7 @@ match.
                                      (end (length target-string))
                                      preserve-case
                                      simple-calls
-                                     (element-type 'character))
+                                     (element-type #+:lispworks 'lw:simple-char #-:lispworks 'character))
   (declare #.*standard-optimize-settings*)
   "Try to match TARGET-STRING between START and END against REGEX and
 replace all matches with REPLACEMENT.
@@ -1157,8 +1163,8 @@ meaningful information about a symbol."
               (push (format nil "[constant]~:[~; value: ~S~]"
                             (boundp symbol) (symbol-value symbol)) output-list))
             ((boundp symbol)
-              (push #+(or LispWorks CLISP) "[variable]"
-                    #-(or LispWorks CLISP) (format nil "[variable] value: ~S"
+              (push #+(or :lispworks :clisp) "[variable]"
+                    #-(or :lispworks :clisp) (format nil "[variable] value: ~S"
                                                    (symbol-value symbol))
                     output-list)))
       #-(or :cormanlisp :clisp)
