@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/closures.lisp,v 1.30 2006/01/03 18:38:55 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/closures.lisp,v 1.32 2006/10/11 15:41:42 edi Exp $
 
 ;;; Here we create the closures which together build the final
 ;;; scanner.
@@ -63,6 +63,7 @@ START-POS, and tests whether REGEX can match *STRING* at START-POS
 such that the call to NEXT-FN after the match would succeed."))
                 
 (defmethod create-matcher-aux ((seq seq) next-fn)
+  (declare #.*standard-optimize-settings*)
   ;; the closure for a SEQ is a chain of closures for the elements of
   ;; this sequence which call each other in turn; the last closure
   ;; calls NEXT-FN
@@ -72,6 +73,7 @@ such that the call to NEXT-FN after the match would succeed."))
         finally (return next-matcher)))
 
 (defmethod create-matcher-aux ((alternation alternation) next-fn)
+  (declare #.*standard-optimize-settings*)
   ;; first create closures for all alternations of ALTERNATION
   (let ((all-matchers (mapcar #'(lambda (choice)
                                   (create-matcher-aux choice next-fn))
@@ -84,6 +86,7 @@ such that the call to NEXT-FN after the match would succeed."))
             thereis (funcall (the function matcher) start-pos)))))
 
 (defmethod create-matcher-aux ((register register) next-fn)
+  (declare #.*standard-optimize-settings*)
   ;; the position of this REGISTER within the whole regex; we start to
   ;; count at 0
   (let ((num (num register)))
@@ -122,6 +125,7 @@ such that the call to NEXT-FN after the match would succeed."))
               next-pos)))))))
 
 (defmethod create-matcher-aux ((lookahead lookahead) next-fn)
+  (declare #.*standard-optimize-settings*)
   ;; create a closure which just checks for the inner regex and
   ;; doesn't care about NEXT-FN
   (let ((test-matcher (create-matcher-aux (regex lookahead) #'identity)))
@@ -139,6 +143,7 @@ such that the call to NEXT-FN after the match would succeed."))
              (funcall next-fn start-pos))))))
 
 (defmethod create-matcher-aux ((lookbehind lookbehind) next-fn)
+  (declare #.*standard-optimize-settings*)
   (let ((len (len lookbehind))
         ;; create a closure which just checks for the inner regex and
         ;; doesn't care about NEXT-FN
@@ -275,6 +280,7 @@ against CHR-EXPR."
                      `(gethash ,chr-expr hash)))))))))
 
 (defmethod create-matcher-aux ((char-class char-class) next-fn)
+  (declare #.*standard-optimize-settings*)
   (declare (type function next-fn))
   ;; insert a test against the current character within *STRING*
   (insert-char-class-tester (char-class (schar *string* start-pos))
@@ -291,6 +297,7 @@ against CHR-EXPR."
              (funcall next-fn (1+ start-pos)))))))
 
 (defmethod create-matcher-aux ((str str) next-fn)
+  (declare #.*standard-optimize-settings*)
   (declare (type fixnum *end-string-pos*)
            (type function next-fn)
            ;; this special value is set by CREATE-SCANNER when the
@@ -405,6 +412,7 @@ against CHR-EXPR."
                   (word-char-p (schar *string* start-pos)))))))
 
 (defmethod create-matcher-aux ((word-boundary word-boundary) next-fn)
+  (declare #.*standard-optimize-settings*)
   (declare (type function next-fn))
   (if (negatedp word-boundary)
     (lambda (start-pos)
@@ -415,6 +423,7 @@ against CHR-EXPR."
            (funcall next-fn start-pos)))))
 
 (defmethod create-matcher-aux ((everything everything) next-fn)
+  (declare #.*standard-optimize-settings*)
   (declare (type function next-fn))
   (if (single-line-p everything)
     ;; closure for single-line-mode: we really match everything, so we
@@ -432,11 +441,12 @@ against CHR-EXPR."
            (funcall next-fn (1+ start-pos))))))
 
 (defmethod create-matcher-aux ((anchor anchor) next-fn)
+  (declare #.*standard-optimize-settings*)
   (declare (type function next-fn))
   (let ((startp (startp anchor))
         (multi-line-p (multi-line-p anchor)))
     (cond ((no-newline-p anchor)
-            ;; this must be and end-anchor and it must be modeless, so
+            ;; this must be an end-anchor and it must be modeless, so
             ;; we just have to check whether START-POS equals
             ;; *END-POS*
             (lambda (start-pos)
@@ -486,6 +496,7 @@ against CHR-EXPR."
                    (funcall next-fn start-pos)))))))
 
 (defmethod create-matcher-aux ((back-reference back-reference) next-fn)
+  (declare #.*standard-optimize-settings*)
   (declare (type function next-fn))
   ;; the position of the corresponding REGISTER within the whole
   ;; regex; we start to count at 0
@@ -525,6 +536,7 @@ against CHR-EXPR."
                    (funcall next-fn next-pos)))))))))
 
 (defmethod create-matcher-aux ((branch branch) next-fn)
+  (declare #.*standard-optimize-settings*)
   (let* ((test (test branch))
          (then-matcher (create-matcher-aux (then-regex branch) next-fn))
          (else-matcher (create-matcher-aux (else-regex branch) next-fn)))
@@ -545,6 +557,7 @@ against CHR-EXPR."
                   (funcall else-matcher start-pos))))))))
 
 (defmethod create-matcher-aux ((standalone standalone) next-fn)
+  (declare #.*standard-optimize-settings*)
   (let ((inner-matcher (create-matcher-aux (regex standalone) #'identity)))
     (declare (type function next-fn inner-matcher))
     (lambda (start-pos)
@@ -553,6 +566,7 @@ against CHR-EXPR."
              (funcall next-fn next-pos))))))
 
 (defmethod create-matcher-aux ((filter filter) next-fn)
+  (declare #.*standard-optimize-settings*)
   (let ((fn (fn filter)))
     (lambda (start-pos)
       (let ((next-pos (funcall fn start-pos)))
@@ -560,5 +574,6 @@ against CHR-EXPR."
              (funcall next-fn next-pos))))))
 
 (defmethod create-matcher-aux ((void void) next-fn)
+  (declare #.*standard-optimize-settings*)
   ;; optimize away VOIDs: don't create a closure, just return NEXT-FN
   next-fn)
