@@ -1,11 +1,11 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/convert.lisp,v 1.22 2005/04/01 21:29:09 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/convert.lisp,v 1.24 2006/07/03 11:16:48 edi Exp $
 
 ;;; Here the parse tree is converted into its internal representation
 ;;; using REGEX objects.  At the same time some optimizations are
 ;;; already applied.
 
-;;; Copyright (c) 2002-2005, Dr. Edmund Weitz. All rights reserved.
+;;; Copyright (c) 2002-2006, Dr. Edmund Weitz. All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -544,6 +544,10 @@ Will also
                                :num (1- backref-number)
                                :case-insensitive-p (case-insensitive-mode-p
                                                     flags))))
+            ;; (:REGEX <string>)
+            ((:regex)
+             (let ((regex (second parse-tree)))
+               (convert-aux (parse-string regex))))
             ;; (:CHAR-CLASS|:INVERTED-CHAR-CLASS {<item>}*)
             ;; where item is one of
             ;;   - a character
